/*
 * $Id: iterator.h,v 1.4 2004/09/04 07:08:01 jklein Exp $
 * $Author: jklein $
 * $Log: iterator.h,v $
 * Revision 1.4  2004/09/04 07:08:01  jklein
 * fixed "const spointer" to "sconstpointer".
 *
 * Revision 1.3  2004/06/28 09:51:09  jklein
 * interface was changed in scale/iterator.h
 *
 * Revision 1.2  2004/06/26 06:58:13  jklein
 * small fix in each header files.
 *
 * Revision 1.1  2004/06/21 16:44:17  jklein
 * Added iterator.
 *
 */

/**
 * siterator_tϳƥꥹȹ¤ؤݲ줿ˡ󶡤롣
 * ƥꥹȹ¤Ϥsiterator_tѰդ뤳ȡ
 * ʲ
 * <pre>
 * siterator_t *itr = s_xxxx_iterator(xxxx);
 * spointer contents;
 * 
 * while ((contents = s_iterator_next(itr))) != NULL) {
 *		'contents'Ѥ
 * }
 *
 * itr->free(itr);
 * </pre>
 * ǽ餫롼פľȤ s_iterator_clear() Ƥ֡
 * ˤꥤƥ졼˺ƽ롣
 * ʤsiterator.dataϤ줾Υƥ졼μ
 * ǻȤǽΤƤΰʤΤǡʤȡ
 */


#ifndef _SCALE_ITERATOR_H_
#define _SCALE_ITERATOR_H_

#include <scale/commons.h>

/** ƥ졼֥ */
typedef struct _sc_iterator siterator_t;

typedef spointer (* s_itr_nextf)(siterator_t *);
typedef void (* s_itr_clearf)(siterator_t *);
typedef void (* s_itr_freef)(siterator_t *);


/** ƥ졼¤ */
struct _sc_iterator {
	spointer listdata;
	spointer data;
	s_itr_nextf next;
	s_itr_clearf clear;
	s_itr_freef free;
};


/** ƥ졼λؤƤǡ֤Υǡ֤򤹤 */
#define s_iterator_next(__itr) (__itr->next(__itr))

/** ƥ졼ƽ */
#define s_iterator_clear(__itr) (__itr->clear(__itr))

/** ƥ졼 */
#define s_iterator_free(__itr) (__itr->free(__itr))


#endif /* _SCALE_ITERATOR_H_ */
